#!/bin/bash
#
# This script emulates the legacy program 'gbkfonts' (by shredder@ctex.org?)
# using fontforge.
#
# TODO: use ttf2pt1 from the `ttf2pt1' package instead of fontforge?
#

texmfmain=`kpsewhich --expand-path='$TEXMFMAIN'`
test -n "$texmfmain" || texmfmain=/usr/share/texmf
sfd=${texmfmain}/tex/latex/ccmap/UGBK.sfd
if ! test -r $sfd; then
    echo 1>&2 "Error: can't open file \"$sfd\", abort."
fi

if ! afm2tfm --help >/dev/null 2>&1; then
    cat <<END 1>&2
$0 requires the 'afm2tfm' program.
Please install either the tetex-afm or the texlive-afm package.
END
    exit 1
fi

# Process cmdline options
fast=false
uid="8000"
name=""
prefix="gbk"
while test $# -gt 0; do
    echo "$1" | grep -q '^-' 2>/dev/null || break
    case "$1" in
	-f)		shift; fast=true;;
	-uid)		shift; uid="$1"; shift;;
	-name)		shift; name="$1"; shift;;
	-prefix)	shift; prefix="$1"; shift;;
	-*)		echo 1>&2 "Invalid option \"$1\""; exit 1;;
    esac
done

# Check remaining cmdline arguments
if test $# -ne 2; then
    cat <<END 2>&1
This bash script simulates the 'gbkfonts' program using fontforge.

Usage:
    $0 [-f] [-uid uid] [-name str] [-prefix str] ttf_file facename

Command line arguments (some options are currently ignored):
    -f:		fast, only create config and map files
    -uid:		select an unique ID for your font
    -name:	select an PostScript font name for you font
    -prefix:	add the prefix rather than "gbk" to you font name
    ttf_file:	input (TTF) file name
    facename:	face name (song/hei/kai/li/fs, etc.)
END
    exit 1
fi

ttf_file="$1"
facename0="$2"
facename=${prefix}"$2"

# make sure the TTF file is readable
if ! test -r "$ttf_file"; then
    echo 1>&2 "Cannot read file \"$ttf_file\", abort."
    exit 1
fi

# Create cid-x.map
echo 1>&2 "*** [GBKfonts] Creating cid-x.map"
cat <<END >cid-x.map
${facename}@UGBK@	UniGB-UCS2-H	:0:`basename "$ttf_file"`
${facename}sl@UGBK@	UniGB-UCS2-H	:0:`basename "$ttf_file"`
END

# Create pdftex.cfg
echo 1>&2 "*** [GBKfonts] Creating pdftex.map"
cat <<END >pdftex.cfg
map +${facename}.map
END

# Create ttfonts.map
echo 1>&2 "*** [GBKfonts] Creating ttfonts.map"
cat <<END >ttfonts.map
${facename}@UGBK@    simsun.ttf  Pid = 3 Eid = 1
${facename}sl@UGBK@    simsun.ttf  Slant = 0.167 Pid = 3 Eid = 1
END

# Create cjk.map
echo 1>&2 "*** [GBKfonts] Creating cjk.map"
/bin/rm -f cjk.map
for ((i=1; i<=94; i++)); do
    fn=`echo $i | awk '{printf "'$facename'%02d", $1}'`
    fnsl=`echo $i | awk '{printf "'$facename'sl%02d", $1}'`
    echo "$fn $fn <${fn}.pfb" >> cjk.map
    echo "$fnsl $fn \" .167 SlantFont \" <${fn}.pfb" >> cjk.map
done

# Create dvipdfm/config/${facename}.map
echo 1>&2 "*** [GBKfonts] Creating dvipdfm/config/${facename}.map"
/bin/rm -rf dvipdfm
mkdir -p dvipdfm/config
for ((i=1; i<=94; i++)); do
    fn=`echo $i | awk '{printf "'$facename'%02d", $1}'`
    fnsl=`echo $i | awk '{printf "'$facename'sl%02d", $1}'`
    echo "$fn none ${fn}" >> dvipdfm/config/${facename}.map
    echo "$fnsl none $fn -s .167" >> dvipdfm/config/${facename}.map
done

# Create tex/latex/CJK/GB/c19${facename0}.fd
echo 1>&2 "*** [GBKfonts] Creating tex/latex/CJK/GB/c19${facename0}.fd"
rm -rf tex
mkdir -p tex/latex/CJK/GB
cat <<END >tex/latex/CJK/GB/c19${facename0}.fd
% Created by gbkfonts.sh
\def\fileversion{4.3.0}
\def\filedate{1999/06/20}

% \ProvidesFile{c19${facename0}.fd}[\filedate\space\fileversion]
% Chinese characters (extension of GB 2312)
% character set: GBK
% font encoding: CJK (extended)

\DeclareFontFamily{C19}{$facename0}{}
\DeclareFontShape{C19}{$facename0}{m}{n}{<-> CJK * ${facename}}{}
\DeclareFontShape{C19}{$facename0}{bx}{n}{<-> CJKb * ${facename}}{}
\DeclareFontShape{C19}{$facename0}{m}{it}{<-> CJK * ${facename}sl}{}
\DeclareFontShape{C19}{$facename0}{bx}{it}{<-> CJKb * ${facename}sl}{}
\DeclareFontShape{C19}{$facename0}{m}{sl}{<-> CJK * ${facename}sl}{}
\DeclareFontShape{C19}{$facename0}{bx}{sl}{<-> CJKb * ${facename}sl}{}

\endinput
END

# Create pdftex/config/${facename}.map
echo 1>&2 "*** [GBKfonts] Creating pdftex/config/${facename}.map"
/bin/rm -rf pdftex
mkdir -p pdftex/config
for ((i=1; i<=94; i++)); do
    fn=`echo $i | awk '{printf "'$facename'%02d", $1}'`
    fnsl=`echo $i | awk '{printf "'$facename'sl%02d", $1}'`
    echo "$fn <${fn}.enc <`basename "$ttf_file"`" \
		>> pdftex/config/${facename}.map
    echo "$fnsl <${fn}.enc <`basename "$ttf_file"`" \
		>> pdftex/config/${facename}.map
done

# Create fonts/{afm,tfm,type1}/chinese/$facename
echo 1>&2 "*** [GBKfonts] Creating .afm and .pfb files (may take a while)"
/bin/rm -rf fonts
mkdir -p fonts/{afm,tfm,type1}/chinese/$facename
cd fonts/type1/chinese/$facename
# create fontforge script file
cat <<END >ttf2pfb.pe
#!/usr/bin/fontforge -script 
#
# Sample usage:
#	mkdir tmp
#	fontforge -script ttf2pfb.pe \\
# 		/usr/share/texmf/tex/latex/ccmap/UGBK.sfd \\
# 		/usr/share/fonts/zh_CN/TrueType/simsun.ttf \\
# 		tmp/gbksong GBK-Song 8000

if (\$argc !=6)
 Print("")
 Print("Usage: ", \$0, " sfd_file ttf_file pfbname fontname uid")
 Print("")
 Quit()
endif

sfd_file = \$1
ttf_file = \$2
pfbname = \$3
fontname = \$4
uid = Strtol(\$5, 10)

Open(ttf_file)
SelectAll()
Generate(pfbname + "%s.pfb", "", -1, -1, sfd_file)
Close()

i = 1
while (i <= 94)
  subfont = "0" + i
  subfont = Strsub(subfont, Strlen(subfont)-2, Strlen(subfont))

  filename = pfbname + subfont + ".pfb"
  #Print("Creating " + filename)

  Open(filename)

  SetFontNames(fontname + subfont)
  SetUniqueID(uid)

  SelectAll()

  Generate(filename)

  Close()

  i = i + 1
endloop
END
# generate fonts
if ! fontforge -script ./ttf2pfb.pe $sfd "$ttf_file" $facename $facename $uid \
	2>fontforge.errlog; then
    cat 1>&2 fontforge.errlog
    /bin/rm -f fontforge.errlog ttf2pfb.pe
    echo 1>&2 "*** [GBKfonts] Warning: error occurred in running fontforge."
###    exit 1
fi
/bin/rm -f fontforge.errlog ttf2pfb.pe
mv *.afm ../../../afm/chinese/${facename}/.
cd ../../../afm/chinese/$facename
echo 1>&2 "*** [GBKfonts] Running afm2tfm to create tfm files"
for f in *.afm; do
    #echo afm2tfm $f ../../../tfm/chinese/${facename}/${f%afm}tfm
    afm2tfm $f ../../../tfm/chinese/${facename}/${f%afm}tfm >/dev/null 2>&1
done

# TODO: create dvips/chinese/${facename}/*.enc
echo 1>&2 "*** [GBKfonts] Warning: .enc files not created (PDFLaTeX won't work)"

echo 1>&2 "*** [GBKfonts] Done"
